var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);

// src/logic/currency/convert.ts
var convertCurrency = (value, from, to, config = {
  EUR: 1,
  CHF: 0.95,
  PLN: 4.5
}) => {
  if (from === to) {
    return value;
  }
  const fromRate = config[from];
  const toRate = config[to];
  const valueInBaseCurrency = value / fromRate;
  return valueInBaseCurrency * toRate;
};

// src/logic/calculator/ProvisionConfigItem.ts
var ProvisionConfigItem = class {
  constructor(currencyRates, provisions) {
    this.currencyRates = currencyRates;
    this.provisions = provisions;
    __publicField(this, "key", "provision");
    __publicField(this, "label", "provision");
    __publicField(this, "dependencies", ["input"]);
  }
  result(input) {
    const provisionConfig = this.provisions.find(
      (config) => input.cost.value <= convertCurrency(
        config.threshold.value,
        config.threshold.currency,
        input.cost.currency,
        this.currencyRates
      )
    );
    if (!provisionConfig) {
      throw new Error("No provision configuration found for the given cost.");
    }
    const provisionValue = input.cost.value * provisionConfig.percentageProvision;
    const minProvisionInInputValue = provisionConfig.staticMinProvision !== void 0 && convertCurrency(
      provisionConfig.staticMinProvision.value,
      provisionConfig.staticMinProvision.currency,
      input.cost.currency,
      this.currencyRates
    );
    const maxProvisionInInputValue = provisionConfig.staticMaxProvision !== void 0 && convertCurrency(
      provisionConfig.staticMaxProvision.value,
      provisionConfig.staticMaxProvision.currency,
      input.cost.currency,
      this.currencyRates
    );
    if (minProvisionInInputValue && provisionValue < minProvisionInInputValue) {
      return {
        value: minProvisionInInputValue,
        currency: input.cost.currency
      };
    }
    if (maxProvisionInInputValue && provisionValue > maxProvisionInInputValue) {
      return {
        value: maxProvisionInInputValue,
        currency: input.cost.currency
      };
    }
    return {
      value: Math.round(provisionValue),
      currency: input.cost.currency
    };
  }
};

// src/logic/calculator/IsCompanyVatConfigItem.ts
var COUNTRY_VAT_RATES = {
  DE: 0.19,
  NL: 0.21,
  PL: 0.23
};
var IsCompanyVatConfigItem = class {
  constructor() {
    __publicField(this, "key", "vat");
    __publicField(this, "label", "vat");
    __publicField(this, "dependencies", ["input", "customs-duty"]);
  }
  result(input) {
    return {
      value: input.value.isCompany ? 0 : COUNTRY_VAT_RATES[input.value.country] * input.cost.value,
      currency: input.cost.currency
    };
  }
};

// src/logic/calculator/BasicCalculator.ts
var BasicCalculator = class {
  constructor(config, currencyRates) {
    this.config = config;
    this.currencyRates = currencyRates;
  }
  static create(config, currencyRates) {
    return new this(config, currencyRates);
  }
  getSumOfDependencies(inputCostCurrency, dependencies, costLines) {
    return dependencies.reduce(
      (cost, configItemKey) => {
        const dependency = costLines.find((f) => f.key === configItemKey);
        if (!dependency) {
          throw new Error(
            `Dependency not found bad config ${dependency} ${JSON.stringify(costLines)}`
          );
        }
        const sameValueCost = convertCurrency(
          dependency.cost.value,
          dependency.cost.currency,
          inputCostCurrency,
          this.currencyRates
        );
        return __spreadProps(__spreadValues({}, cost), {
          value: sameValueCost + cost.value
        });
      },
      { currency: inputCostCurrency, value: 0 }
    );
  }
  getFinalCost(cost, values) {
    const costLines = [
      {
        key: "input",
        cost
      }
    ];
    this.config.forEach((config) => {
      var _a;
      let result = null;
      const dependenciesSum = this.getSumOfDependencies(
        cost.currency,
        config.dependencies,
        costLines
      );
      switch (config.key) {
        case "vat": {
          result = config.result({
            cost: dependenciesSum,
            value: {
              isCompany: values.isImportedFromEu ? true : values.isCompany,
              // in ue you don't pay extra vat
              country: values.customDutyCountry
            }
          });
          break;
        }
        case "transport":
        case "excise-duty": {
          result = config.result({
            cost: dependenciesSum,
            value: {
              engineOver20CCM: values.engineOver20CCM,
              type: values.vehicleType
            }
          });
          break;
        }
        case "customs-duty":
          result = config.result({
            cost: dependenciesSum,
            value: {
              type: values.vehicleType,
              isOutsideEu: !values.isImportedFromEu && values.isManufacturedOutsideEu
            }
          });
          break;
        case "extra-costs": {
          const isChecked = (_a = values.extraCosts.find(
            (f) => f.label === config.label
          )) == null ? void 0 : _a.checked;
          result = config.result({
            cost,
            value: Boolean(isChecked)
          });
          break;
        }
        case "provision":
          result = config.result({
            cost,
            value: void 0
          });
          break;
        default:
          console.warn(`Config not handled ${config.key}`);
      }
      if (result) {
        costLines.push({
          cost: {
            value: result.value,
            currency: result.currency
          },
          key: config.key,
          label: result.label
        });
      }
    });
    return {
      costLines,
      finalCost: {
        currency: cost.currency,
        value: costLines.reduce((a, b) => a + b.cost.value, 0)
      }
    };
  }
};

// src/logic/calculator/ExtraCostsConfigItem.ts
var ExtraCostConfigItem = class {
  constructor(currencyRates, extraCost, label) {
    this.currencyRates = currencyRates;
    this.extraCost = extraCost;
    this.label = label;
    __publicField(this, "key", "extra-costs");
    __publicField(this, "dependencies", ["input"]);
  }
  result(input) {
    if (input.value) {
      const provisionAsInputCurrency = convertCurrency(
        this.extraCost.value,
        this.extraCost.currency,
        input.cost.currency,
        this.currencyRates
      );
      return {
        label: this.label,
        value: provisionAsInputCurrency,
        currency: input.cost.currency
      };
    }
    return {
      value: 0,
      currency: input.cost.currency,
      label: this.label
    };
  }
};

// src/logic/calculator/HasCustomDutyConfigItem.ts
var CustomDutyExciseStandard = /* @__PURE__ */ ((CustomDutyExciseStandard2) => {
  CustomDutyExciseStandard2[CustomDutyExciseStandard2["STANDARD"] = 0.1] = "STANDARD";
  return CustomDutyExciseStandard2;
})(CustomDutyExciseStandard || {});
var HasCustomDutyConfigItem = class {
  constructor(customDutyExcisePerVehicleType = {
    STANDARD: 0.1 /* STANDARD */,
    MOTORCYCLE: 0.1 /* STANDARD */
  }, dependencies = ["input"]) {
    this.customDutyExcisePerVehicleType = customDutyExcisePerVehicleType;
    this.dependencies = dependencies;
    __publicField(this, "key", "customs-duty");
    // cło
    __publicField(this, "label", "customsDuty");
  }
  result(input) {
    if (input.value.isOutsideEu) {
      return {
        value: (input.value.type === "MOTORCYCLE" ? this.customDutyExcisePerVehicleType.MOTORCYCLE : this.customDutyExcisePerVehicleType.STANDARD) * input.cost.value,
        currency: input.cost.currency
      };
    }
    return {
      value: 0,
      currency: input.cost.currency
    };
  }
};

// src/logic/calculator/HasExciseDutyConfigItem.ts
var HasExciseDutyConfigItem = class {
  constructor() {
    __publicField(this, "key", "excise-duty");
    // akcyza
    __publicField(this, "label", "exciseDuty");
    __publicField(this, "dependencies", ["input", "customs-duty", "vat"]);
  }
  result(input) {
    const {
      value: { type, engineOver20CCM }
    } = input;
    let exciseRate;
    switch (type) {
      case "CAR":
        exciseRate = engineOver20CCM ? 0.186 /* VEHICLE_OVER_2_ENGINE */ : 0.031 /* VEHICLE_STANDARD */;
        break;
      case "MOTORCYCLE":
        exciseRate = 0 /* ZERO */;
        break;
      case "TRUCK":
        exciseRate = 0.186 /* VEHICLE_OVER_2_ENGINE */;
        break;
      case "ELECTRIC_CAR":
        exciseRate = 0 /* ZERO */;
        break;
      case "HYBRID_PLUG_IN":
        exciseRate = 0 /* ZERO */;
        break;
      case "HYBRID_CAR":
        if (engineOver20CCM) {
          exciseRate = 0.093 /* HYBRID_LESS_2_ENGINE */;
        } else {
          exciseRate = 0.0155 /* HYBRID_OVER_2_ENGINE */;
        }
        break;
      default:
        exciseRate = 0 /* ZERO */;
        break;
    }
    return {
      value: input.cost.value * exciseRate,
      currency: input.cost.currency
    };
  }
};

// src/logic/calculator/TransportConfigItem.ts
var TransportConfigItem = class {
  constructor(currencyRates, provision) {
    this.currencyRates = currencyRates;
    this.provision = provision;
    __publicField(this, "key", "transport");
    __publicField(this, "label", "transport");
    __publicField(this, "dependencies", ["input"]);
  }
  result(input) {
    const {
      value: { type }
    } = input;
    const provisionCost = this.provision[type];
    const provisionAsInputCurrency = convertCurrency(
      provisionCost.value,
      provisionCost.currency,
      input.cost.currency,
      this.currencyRates
    );
    return {
      value: provisionAsInputCurrency,
      currency: input.cost.currency
    };
  }
};

// src/index.ts
var Calculator = {
  ProvisionConfigItem,
  IsCompanyVatConfigItem,
  ExtraCostConfigItem,
  HasCustomDutyConfigItem,
  HasExciseDutyConfigItem,
  TransportConfigItem,
  BasicCalculator,
  CustomDutyExciseStandard
};
var Currency = {
  convertCurrency
};
export {
  Calculator,
  Currency
};
